/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2008, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.maven.plugins.as.common;

import java.io.File;
import java.io.IOException;

import org.apache.commons.io.FileUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jboss.jbossas.servermanager.Server;

/**
 * JbossAsConfigurationMojoBase
 * 
 * Support for JBossAS Configuration Mojos
 * 
 * @author <a href="mailto:andrew.rubinger@jboss.org">ALR</a>
 * @version $Revision: $
 */
public class JbossAsConfigurationDelegate extends JbossAsBaseDelegate
{
   // Class Members

   private static final Log logger = LogFactory.getLog(JbossAsConfigurationDelegate.class);

   // Functional Methods

   /**
    * Creates a server configuration of the specified name by copying the
    * specified base, and overlaying configuration files from the specified
    * overlayLocation.  If the configuration
    * at "configName" already exists, it will be overwritten if the overwrite flag
    * is set.  If no "configBase" is set, this method will return with no action (as
    * it's assumed the configuration exists)
    *  
    * @param jbossHome
    * @param configName
    * @param configBase
    * @param overwrite
    * @param overlayLocation
    * @exception IOException
    */
   public void createServerConfiguration(String jbossHome, String configName, String configBase,
         String overlayLocation, boolean overwrite) throws IOException
   {
      // If no base is defined, return, server configuration assumed exists
      if (configBase == null || configBase.equals(""))
      {
         logger.debug("No configuration base defined while creating server configuration, "
               + "assumed exists, no new config will be made.");
         return;
      }

      // Copy the existing configurations
      this.copyServerConfiguration(jbossHome, configName, configBase, overwrite);

      // Overlay
      this.overlayConfiguration(jbossHome, configName, overlayLocation);
   }

   /**
    * Overlays the files from the specified overlayLocation on top of the specified 
    * JBoss AS configuration located [jbossHome]/server/configName.  If overlayLocation is null,
    * no action will be performed.
    * 
    * @param jbossHome
    * @param configName
    * @param overlayLocation
    * @throws IOException
    */
   public void overlayConfiguration(String jbossHome, String configName, String overlayLocation) throws IOException
   {
      // Check JBOSS_HOME
      this.checkJbossHome();

      // If no overlay location, gracefully exit
      if (overlayLocation == null || overlayLocation.equals(""))
      {
         return;
      }

      // Obtain reference to Overlay Location
      File overlayDir = new File(overlayLocation);

      // Ensure Overlay Location is a valid directory
      if (!overlayDir.exists() || !overlayDir.isDirectory())
      {
         throw new RuntimeException("Specified Overlay Location must be a valid directory; \"" + overlayLocation
               + "\" is not.");
      }

      // Concat Configuration Path
      String configPath = jbossHome + File.separator + Server.JBOSS_SERVER_CONFIG_DIR_NAME
            + File.separator + configName;

      // Obtain reference to Configuration Path
      File configDir = new File(configPath);

      // Ensure Config is a valid directory
      if (!configDir.exists() || !configDir.isDirectory())
      {
         throw new RuntimeException("Specified JBossAS configuration must be a valid directory; \"" + configPath
               + "\" is not.");
      }

      JbossAsConfigurationDelegate.logger.info("Overlaying configuration from " + overlayLocation + " to " + configPath
            + "...");

      // Overlay
      FileUtils.copyDirectory(overlayDir, configDir, false);
   }

   /**
    * Copies the existing server configuration from [jbossHome]/server/[configBase] to
    * a new configuration at [jbossHome]/server/[configName].  If the new configuration
    * name specified exists and the overwrite flag is set, 
    * the new configuration will be overwritten.  If the configBase is null,
    * no action will be taken. 
    * 
    * @param jbossHome
    * @param configName
    * @param configBase
    * @param overwrite
    */
   public void copyServerConfiguration(String jbossHome, String configName, String configBase, boolean overwrite)
         throws IOException
   {
      // Check JBOSS_HOME
      this.checkJbossHome();

      // If base is not defined, this isn't a true copy request, return
      if (configBase == null)
      {
         return;
      }

      // Concat configuration paths
      String configPath = jbossHome + File.separator + Server.JBOSS_SERVER_CONFIG_DIR_NAME
            + File.separator;
      String baseConfigPath = configPath + configBase;
      String destConfigPath = configPath + configName;

      // Obtain Base Configuration
      File baseConfig = new File(baseConfigPath);

      // Ensure base exists
      if (!baseConfig.exists())
      {
         throw new RuntimeException("The specified base configuration at " + baseConfigPath + " does not exist.");
      }

      // Create destination
      File destConfig = new File(destConfigPath);

      // If destination already exists, remove
      if (destConfig.exists() && overwrite)
      {
         FileUtils.deleteDirectory(destConfig);
      }

      // Copy
      FileUtils.copyDirectory(baseConfig, destConfig);

      // Log
      JbossAsConfigurationDelegate.logger.info("Copied JBoss Base Server Configuration from " + baseConfigPath + " to "
            + destConfigPath);

   }
   
}
