/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2008, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.maven.plugins.as.common;

import java.io.File;
import java.io.IOException;
import java.util.Collection;
import java.util.HashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jboss.jbossas.servermanager.Argument;
import org.jboss.jbossas.servermanager.Server;
import org.jboss.jbossas.servermanager.ServerManager;

/**
 * JbossAsRuntimeMojoBase
 * 
 * Support for JBossAS Runtime Mojos (Requiring Start/Stop, etc)
 * 
 * @author <a href="mailto:andrew.rubinger@jboss.org">ALR</a>
 * @version $Revision: $
 */
public class JbossAsRuntimeDelegate extends JbossAsBaseDelegate
{
   // Class Members

   private static final Log logger = LogFactory.getLog(JbossAsRuntimeDelegate.class);

   public static final String ENV_VAR_JAVA_HOME = "JAVA_HOME";

   private static String javaHome;

   private static ServerManager serverManager;

   // Functional Methods

   /**
    * Starts the Application Server
    * 
    * @param serverConfigName
    * @param jvmargs
    * @return The unique name of the Server started
    * @exception IOException
    */
   public String startJbossAs(String serverConfigName, Collection<String> jvmargs) throws IOException
   {
      Server server = setupServer(serverConfigName, jvmargs);

      ServerManager serverManager = setupServerManager(server);

      // Start the Server
      serverManager.startServer(server.getName());

      // Return the name of the Server we've started
      return server.getName();
   }

   /**
    * Configures the Server Manager and add's the specified server to it.
    * 
    * @param server
    * @return
    * @throws IOException
    */
   private ServerManager setupServerManager(Server server) throws IOException
   {
      // Create ServerManager
      ServerManager serverManager = this.getServerManager();

      // Set JVM / JBOSS_HOME
      serverManager.setJavaHome(this.getJavaHome());
      serverManager.setJbossHome(this.getJbossHome());

      // Add a Server
      serverManager.addServer(server);

      return serverManager;
   }

   /**
    * Instantiates a Server instance from the given server name.
    * 
    * @param serverConfigName
    * @param jvmargs
    * @return
    */
   private Server setupServer(String serverConfigName, Collection<String> jvmargs)
   {
      // Sanity checks
      this.checkJbossHome();
      this.checkJavaHome();

      Collection<Argument> arguments = this.createJvmArguments(jvmargs);

      // Ensure configuration is set
      if (serverConfigName == null || serverConfigName.equals(""))
      {
         throw new RuntimeException("Configuration Parameter serverConfigName must be set before running the plugin.");
      }

      // Log 
      JbossAsRuntimeDelegate.logger.info("Using $" + JbossAsRuntimeDelegate.ENV_VAR_JAVA_HOME + ": "
            + this.getJavaHome());
      JbossAsRuntimeDelegate.logger.info("Using $" + JbossAsRuntimeDelegate.ENV_VAR_JBOSS_HOME + ": "
            + this.getJbossHome());
      JbossAsRuntimeDelegate.logger.info("Using Server Configuration: " + serverConfigName);
      JbossAsRuntimeDelegate.logger.info("Using JVM Arguments: " + jvmargs == null ? "[NONE]" : jvmargs);

      // Create Server
      Server server = new Server();
      server.setName(serverConfigName);

      // Set JVM Arguments
      if (arguments != null)
      {
         for (Argument arg : arguments)
         {
            server.addJvmArg(arg);
         }
      }
      return server;
   }

   /**
    * Stops the specified server with the specified registered name
    * 
    * @param serverName
    * @exception IOException
    */
   public void stopJbossAs(String serverName) throws IOException
   {
      // Delegate to ServerManager to stop the server
      this.getServerManager().stopServer(serverName);
   }

   /**
    * Ensure $JAVA_HOME is defined
    */
   public void checkJavaHome()
   {
      // Ensure Environment Variable JAVA_HOME is set
      if (this.getJavaHome() == null || this.getJavaHome().equals(""))
      {
         String errorMessage = "Environment Variable \"" + JbossAsRuntimeDelegate.ENV_VAR_JAVA_HOME
               + "\" must be set before running the plugin.";
         throw new RuntimeException(errorMessage, new IllegalStateException(errorMessage));
      }
   }

   // Internal Helper Methods

   /**
    * Create proper Argument objects from the
    * Strings passed into the Plugin configuration
    */
   private Collection<Argument> createJvmArguments(Collection<String> jvmargs)
   {
      // Initialize
      Collection<Argument> args = new HashSet<Argument>();

      if (jvmargs != null)
      {
         for (String arg : jvmargs)
         {
            // Create new Argument
            Argument a = new Argument();

            // Set Value
            a.setValue(arg);

            // Add to List
            args.add(a);
         }
      }
      else
      {
         return null;
      }

      // Return
      return args;

   }

   public String getJavaHome()
   {
      if (JbossAsRuntimeDelegate.javaHome == null || JbossAsRuntimeDelegate.javaHome.equals(""))
      {
         JbossAsRuntimeDelegate.javaHome = System.getenv(JbossAsRuntimeDelegate.ENV_VAR_JAVA_HOME);
      }

      return javaHome;
   }

   public ServerManager getServerManager()
   {
      if (JbossAsRuntimeDelegate.serverManager == null)
      {
         JbossAsRuntimeDelegate.serverManager = new ServerManager();
      }

      return JbossAsRuntimeDelegate.serverManager;
   }

   /**
    * Deploys the artifact to the named server.
    * 
    * @param serverConfigName
    * @param jvmargs
    * @param artifact
    * @throws IOException
    */
   public void deploy(String serverConfigName, Collection<String> jvmargs, File artifact) throws IOException
   {
      // Obtain the server
      Server server = getServer(serverConfigName, jvmargs);

      // Deploy
      try
      {
         server.deploy(artifact);
      }
      catch (Exception e)
      {
         throw new RuntimeException("Could not deploy file: " + artifact, e);
      }
   }

   /**
    * Undeployes the artifact from the named server.
    * 
    * @param serverConfigName
    * @param jvmargs
    * @param artifact
    * @throws IOException
    */
   public void undeploy(String serverConfigName, Collection<String> jvmargs, File artifact) throws IOException
   {
      // Obtain the server
      Server server = getServer(serverConfigName, jvmargs);

      // Undeploy
      try
      {
         server.undeploy(artifact);
      }
      catch (Exception e)
      {
         throw new RuntimeException("Could not undeploy file: " + artifact, e);
      }
   }

   /**
    * Given a server name, checks with the server manager and returns an existing instance, or 
    * returns a newly created one.
    * 
    * @param serverConfigName
    * @param jvmargs
    * @return an existing or new server instance.
    * @throws IOException
    */
   private Server getServer(String serverConfigName, Collection<String> jvmargs) throws IOException
   {
      // the server instance we will work with
      Server server;

      // is there an existing server instance?
      try
      {
         server = this.getServerManager().getServer(serverConfigName);
      }
      catch (IllegalArgumentException e)
      {
         // server with name serverConfigName doesn't exist
         //try to obtain instance
         server = setupServer(serverConfigName, jvmargs);

         // setup the server manager
         setupServerManager(server);
      }
      return server;
   }

}
